//---------------------------------------------------------------------------
//
// *** formsC unit ***
// (TFormC, TFormTaskbarC, TMainFormC, TMonitorList)
//
// by Arash Ramin - http://www.digitalroom.net
// version 1.20 (August 09, 2001)
//
// (see formsC.txt for documentation)
//---------------------------------------------------------------------------
#ifndef formsCH
#define formsCH
//---------------------------------------------------------------------------
#include <multimon.h>

#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>

#include <stdlib.h>
#include <windef.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------

/************************************************************************/
/* VERSION CHECK ********************************************************/
/************************************************************************/

/* C++Builder 5 onwards fixes the animation and system sound problem when
   minimizing and restoring the main form.  The code pertaining to this
   should only be compiled for C++Builder 1/3/4. */
#if ((__BORLANDC__ == 0x520)||(__BORLANDC__ == 0x530)||(__BORLANDC__ == 0x540))
#define MAINFORM_FIX
#endif

/************************************************************************/
/* formsC - GLOBAL DECLARATIONS *****************************************/
/************************************************************************/

//---------------------------------------------------------------------------
extern TPoint __fastcall CheckPosition(HWND hWnd);
extern TPoint __fastcall CenterForm(HWND hWnd, HWND hParent);

#ifdef MAINFORM_FIX
extern void __fastcall AppMinimize();
extern void __fastcall AppRestore();
#endif
//---------------------------------------------------------------------------

/************************************************************************/
/* TFormC - CLASS DECLARATION *******************************************/
/************************************************************************/

class TFormC : public TForm
{
  typedef TForm inherited;

public:

  void __fastcall SetOnTop(bool OnTop);

  HIDESBASE void __fastcall Show();
  int __fastcall ShowModal();
  HIDESBASE void __fastcall ShowCenter(HWND hParent);
  int __fastcall ShowModalCenter(HWND hParent);

  __fastcall virtual TFormC(TComponent* AOwner) : inherited(AOwner) {};
  __fastcall TFormC(TComponent* AOwner, int Dummy)
      : inherited(AOwner, Dummy) {};
  __fastcall TFormC(HWND ParentWindow) : inherited(ParentWindow) {};
  __fastcall virtual ~TFormC() {};

};
//---------------------------------------------------------------------------

/************************************************************************/
/* TFormTaskbarC - CLASS DECLARATION ************************************/
/************************************************************************/

class TFormTaskbarC : public TFormC
{
  typedef TFormC inherited;

protected:
  void __fastcall CreateParams(TCreateParams &Params);

public:

  __fastcall virtual TFormTaskbarC(TComponent* AOwner) : inherited(AOwner) {};
  __fastcall virtual ~TFormTaskbarC() {};

};
//---------------------------------------------------------------------------


/************************************************************************/
/* TMainFormC - CLASS DECLARATION ***************************************/
/************************************************************************/

class TMainFormC : public TFormC
{
  typedef TFormC inherited;

private:

  TMessageEvent FOnAppMessage;
  bool FMinimizeOnClose;

protected:

  BEGIN_MESSAGE_MAP
    MESSAGE_HANDLER(WM_SYSCOMMAND,TWMSysCommand,WMSysCommand);
  END_MESSAGE_MAP(TFormC)

  void __fastcall AppMessageC(MSG &Msg, bool &Handled);
  void __fastcall WMSysCommand(TWMSysCommand &Message);

public:

  __fastcall virtual TMainFormC(TComponent* AOwner);
  __fastcall virtual ~TMainFormC() {};

  __property TMessageEvent OnAppMessage =
      {read=FOnAppMessage, write=FOnAppMessage};
  __property bool MinimizeOnClose =
      {read=FMinimizeOnClose, write=FMinimizeOnClose};

};
//---------------------------------------------------------------------------

/************************************************************************/
/* TMonitorList - GLOBAL DECLARATIONS ***********************************/
/************************************************************************/

//---------------------------------------------------------------------------
// callback function for EnumDisplayMonitors
BOOL CALLBACK MonitorEnumProc (
  HMONITOR hMonitor,  // handle to display monitor
  HDC hdcMonitor,     // handle to monitor-appropriate device context
  LPRECT lprcMonitor, // pointer to monitor intersection rectangle
  LPARAM dwData       // data passed from EnumDisplayMonitors
);
//---------------------------------------------------------------------------

/************************************************************************/
/* TMonitorList - DATA STRUCTURES ***************************************/
/************************************************************************/

struct monitorInfo {
  HMONITOR Handle;
  RECT ScreenArea;
  RECT WorkArea;
};

/************************************************************************/
/* TMonitorList - CLASS DECLARATION *************************************/
/************************************************************************/

class TMonitorList
{

  friend BOOL CALLBACK MonitorEnumProc(HMONITOR hMonitor, HDC hdcMonitor,
    LPRECT lprcMonitor, LPARAM dwData);

private:

  TList *FMonitorList;

protected:

  int __fastcall GetCount();
  monitorInfo * __fastcall GetMonInfo(int Index);

public:

  TMonitorList();
  ~TMonitorList();

  __property int Count = {read = GetCount};
  __property monitorInfo *Monitor[int Index] = {read = GetMonInfo};

};
//---------------------------------------------------------------------------
extern TMonitorList *Monitors;
//---------------------------------------------------------------------------
#endif
